@@ -6,9 +6,11 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
 use Carp qw(croak);
 use Config;
-use Exporter ();
-require File::Basename;
-require File::Spec;
+use Cwd            ();
+use Exporter       ();
+use FindBin        ();
+use File::Basename ();
+use File::Spec     ();
 
 =head1 NAME
 
@@ -16,20 +18,20 @@ File::ConfigDir - Get directories of configuration files
 
 =cut
 
-$VERSION = '0.015';
-@ISA     = qw(Exporter);
-@EXPORT  = ();
+$VERSION   = '0.016';
+@ISA       = qw(Exporter);
+@EXPORT    = ();
 @EXPORT_OK = (
-               qw(config_dirs system_cfg_dir desktop_cfg_dir),
-               qw(xdg_config_dirs machine_cfg_dir),
-               qw(core_cfg_dir site_cfg_dir vendor_cfg_dir),
-               qw(locallib_cfg_dir local_cfg_dir),
-               qw(here_cfg_dir singleapp_cfg_dir),
-               qw(xdg_config_home user_cfg_dir)
-             );
+    qw(config_dirs system_cfg_dir desktop_cfg_dir),
+    qw(xdg_config_dirs machine_cfg_dir),
+    qw(core_cfg_dir site_cfg_dir vendor_cfg_dir),
+    qw(locallib_cfg_dir local_cfg_dir),
+    qw(here_cfg_dir singleapp_cfg_dir),
+    qw(xdg_config_home user_cfg_dir)
+);
 %EXPORT_TAGS = (
-                 ALL => [@EXPORT_OK],
-               );
+    ALL => [@EXPORT_OK],
+);
 
 my $haveFileHomeDir = 0;
 eval {
@@ -103,7 +105,7 @@ sub _find_common_base_dir
         push( @commondir, $dirsa[$i] );
     }
 
-    return File::Spec->catdir( $va, @commondir );
+    File::Spec->catdir( $va, @commondir );
 }
 
 =head2 system_cfg_dir
@@ -116,30 +118,21 @@ the value of the environment variable C<%windir%>.
 
 my $system_cfg_dir = sub {
     my @cfg_base = @_;
-    my @dirs;
-    if ( $^O eq "MSWin32" )
-    {
-        push( @dirs, $ENV{windir} );
-    }
-    else
-    {
-        push( @dirs, File::Spec->catdir( "/etc", @cfg_base ) );
-    }
-    return @dirs;
+    my @dirs = File::Spec->catdir( $^O eq "MSWin32" ? $ENV{windir} : "/etc", @cfg_base );
+    @dirs;
 };
 
 sub system_cfg_dir
 {
     my @cfg_base = @_;
     1 < scalar(@cfg_base)
-      and croak "system_cfg_dir(;\$), not system_cfg_dir("
-      . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
-    return &{$system_cfg_dir}(@cfg_base);
+      and croak "system_cfg_dir(;\$), not system_cfg_dir(" . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
+    $system_cfg_dir->(@cfg_base);
 }
 
 =head2 machine_cfg_dir
 
-Alias for desktop_cfg_dir - depreciated.
+Alias for desktop_cfg_dir - deprecated.
 
 =head2 xdg_config_dirs
 
@@ -161,7 +154,7 @@ my $desktop_cfg_dir = sub {
     {
         my $alluserprof = $ENV{ALLUSERSPROFILE};
         my $appdatabase = File::Basename::basename( $ENV{APPDATA} );
-        push( @dirs, File::Spec->catdir( $alluserprof, $appdatabase, @cfg_base ) );
+        @dirs = ( File::Spec->catdir( $alluserprof, $appdatabase, @cfg_base ) );
     }
     else
     {
@@ -172,30 +165,24 @@ my $desktop_cfg_dir = sub {
         }
         else
         {
-            push( @dirs, File::Spec->catdir( "/etc", "xdg", @cfg_base ) );
+            @dirs = ( File::Spec->catdir( "/etc", "xdg", @cfg_base ) );
         }
     }
-    return @dirs;
+    @dirs;
 };
 
 sub desktop_cfg_dir
 {
     my @cfg_base = @_;
     1 < scalar(@cfg_base)
-      and croak "desktop_cfg_dir(;\$), not desktop_cfg_dir("
-      . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
-    return &{$desktop_cfg_dir}(@cfg_base);
+      and croak "desktop_cfg_dir(;\$), not desktop_cfg_dir(" . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
+    $desktop_cfg_dir->(@cfg_base);
 }
 
-sub machine_cfg_dir
-{
-    goto \&desktop_cfg_dir;
-}
-
-sub xdg_config_dirs
-{
-    goto \&desktop_cfg_dir;
-}
+no warnings 'once';
+*machine_cfg_dir = \&desktop_cfg_dir;
+*xdg_config_dirs = \&desktop_cfg_dir;
+use warnings;
 
 =head2 core_cfg_dir
 
@@ -205,19 +192,16 @@ Returns the C<etc> directory below C<$Config{prefix}>.
 
 my $core_cfg_dir = sub {
     my @cfg_base = @_;
-    my @dirs;
-
-    push( @dirs, File::Spec->catdir( $Config{prefix}, "etc", @cfg_base ) );
-    return @dirs;
+    my @dirs = ( File::Spec->catdir( $Config{prefix}, "etc", @cfg_base ) );
+    @dirs;
 };
 
 sub core_cfg_dir
 {
     my @cfg_base = @_;
     1 < scalar(@cfg_base)
-      and croak "core_cfg_dir(;\$), not core_cfg_dir("
-      . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
-    return &{$core_cfg_dir}(@cfg_base);
+      and croak "core_cfg_dir(;\$), not core_cfg_dir(" . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
+    $core_cfg_dir->(@cfg_base);
 }
 
 =head2 site_cfg_dir
@@ -241,16 +225,15 @@ my $site_cfg_dir = sub {
         push( @dirs, File::Spec->catdir( $sitelib_stem, "etc", @cfg_base ) );
     }
 
-    return @dirs;
+    @dirs;
 };
 
 sub site_cfg_dir
 {
     my @cfg_base = @_;
     1 < scalar(@cfg_base)
-      and croak "site_cfg_dir(;\$), not site_cfg_dir("
-      . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
-    return &{$site_cfg_dir}(@cfg_base);
+      and croak "site_cfg_dir(;\$), not site_cfg_dir(" . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
+    $site_cfg_dir->(@cfg_base);
 }
 
 =head2 vendor_cfg_dir
@@ -274,46 +257,40 @@ my $vendor_cfg_dir = sub {
         push( @dirs, File::Spec->catdir( $vendorlib_stem, "etc", @cfg_base ) );
     }
 
-    return @dirs;
+    @dirs;
 };
 
 sub vendor_cfg_dir
 {
     my @cfg_base = @_;
     1 < scalar(@cfg_base)
-      and croak "vendor_cfg_dir(;\$), not vendor_cfg_dir("
-      . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
-    return &{$vendor_cfg_dir}(@cfg_base);
+      and croak "vendor_cfg_dir(;\$), not vendor_cfg_dir(" . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
+    $vendor_cfg_dir->(@cfg_base);
 }
 
 =head2 singleapp_cfg_dir
 
 Returns the configuration file for standalone installed applications. In
-Unix speak, installing JRE to C<< /usr/local/jre-<version> >> means there is a
-C<< /usr/local/jre-<version>/bin/java >> and going from it's directory name
-one above and into C<etc> there is the I<singleapp_cfg_dir>. For a Perl
-module it means, we're assuming that C<$0> is installed as a standalone
-package somewhere, eg. into C</usr/pkg> - as recommended for pkgsrc ;)
+Unix speak, installing JRE to C<< /usr/local/jre-<version> >> means there is
+a C<< /usr/local/jre-<version>/bin/java >> and going from it's directory
+name one above and into C<etc> there is the I<singleapp_cfg_dir>. For a
+Perl module it means, we're assuming that C<$FindBin::Bin> is installed as
+a standalone package somewhere, eg. into C</usr/pkg> - as recommended for
+pkgsrc ;)
 
 =cut
 
 my $singleapp_cfg_dir = sub {
-    my @dirs;
-
-    my $appbin = File::Basename::dirname($0);
-    my $appdir = File::Basename::dirname($appbin);
-    push( @dirs, File::Spec->catdir( $appdir, "etc" ) );
-
-    return @dirs;
+    my @dirs = ( Cwd::abs_path( File::Spec->catdir( $FindBin::RealDir, "..", "etc" ) ) );
+    @dirs;
 };
 
 sub singleapp_cfg_dir
 {
     my @cfg_base = @_;
     0 == scalar(@cfg_base)
-      or croak "singleapp_cfg_dir(), not singleapp_cfg_dir("
-      . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
-    return &{$singleapp_cfg_dir}();
+      or croak "singleapp_cfg_dir(), not singleapp_cfg_dir(" . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
+    $singleapp_cfg_dir->();
 }
 
 =head2 local_cfg_dir
@@ -333,16 +310,15 @@ my $local_cfg_dir = sub {
         push( @dirs, File::Spec->catdir( "/usr", "local", "etc", @cfg_base ) );
     }
 
-    return @dirs;
+    @dirs;
 };
 
 sub local_cfg_dir
 {
     my @cfg_base = @_;
     1 < scalar(@cfg_base)
-      and croak "local_cfg_dir(;\$), not local_cfg_dir("
-      . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
-    return &{$local_cfg_dir}(@cfg_base);
+      and croak "local_cfg_dir(;\$), not local_cfg_dir(" . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
+    $local_cfg_dir->(@cfg_base);
 }
 
 =head2 locallib_cfg_dir
@@ -356,24 +332,23 @@ my $locallib_cfg_dir = sub {
     my @cfg_base = @_;
     my @dirs;
 
-    if (    $INC{'local/lib.pm'}
-         && $ENV{PERL_MM_OPT}
-         && $ENV{PERL_MM_OPT} =~ m/.*INSTALL_BASE=([^"']*)['"]?$/ )
+    if (   $INC{'local/lib.pm'}
+        && $ENV{PERL_MM_OPT}
+        && $ENV{PERL_MM_OPT} =~ m/.*INSTALL_BASE=([^"']*)['"]?$/ )
     {
         ( my $cfgdir = $ENV{PERL_MM_OPT} ) =~ s/.*INSTALL_BASE=([^"']*)['"]?$/$1/;
         push( @dirs, File::Spec->catdir( $cfgdir, "etc", @cfg_base ) );
     }
 
-    return @dirs;
+    @dirs;
 };
 
 sub locallib_cfg_dir
 {
     my @cfg_base = @_;
     1 < scalar(@cfg_base)
-      and croak "locallib_cfg_dir(;\$), not locallib_cfg_dir("
-      . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
-    return &{$locallib_cfg_dir}(@cfg_base);
+      and croak "locallib_cfg_dir(;\$), not locallib_cfg_dir(" . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
+    $locallib_cfg_dir->(@cfg_base);
 }
 
 =head2 here_cfg_dir
@@ -384,21 +359,16 @@ Returns the path for the C<etc> directory below the current working directory.
 
 my $here_cfg_dir = sub {
     my @cfg_base = @_;
-    my @dirs;
-
-    push( @dirs,
-          File::Spec->catdir( File::Spec->rel2abs( File::Spec->curdir() ), @cfg_base, "etc" ) );
-
-    return @dirs;
+    my @dirs = ( File::Spec->catdir( File::Spec->rel2abs( File::Spec->curdir() ), @cfg_base, "etc" ) );
+    @dirs;
 };
 
 sub here_cfg_dir
 {
     my @cfg_base = @_;
     1 < scalar(@cfg_base)
-      and croak "here_cfg_dir(;\$), not here_cfg_dir("
-      . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
-    return &{$here_cfg_dir}(@cfg_base);
+      and croak "here_cfg_dir(;\$), not here_cfg_dir(" . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
+    $here_cfg_dir->(@cfg_base);
 }
 
 =head2 user_cfg_dir
@@ -412,19 +382,17 @@ my $user_cfg_dir = sub {
     my @cfg_base = @_;
     my @dirs;
 
-    push( @dirs, File::Spec->catdir( File::HomeDir->my_home(), map { "." . $_ } @cfg_base ) )
-      if ($haveFileHomeDir);
+    $haveFileHomeDir and @dirs = ( File::Spec->catdir( File::HomeDir->my_home(), map { "." . $_ } @cfg_base ) );
 
-    return @dirs;
+    @dirs;
 };
 
 sub user_cfg_dir
 {
     my @cfg_base = @_;
     1 < scalar(@cfg_base)
-      and croak "user_cfg_dir(;\$), not user_cfg_dir("
-      . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
-    return &{$user_cfg_dir}(@cfg_base);
+      and croak "user_cfg_dir(;\$), not user_cfg_dir(" . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
+    $user_cfg_dir->(@cfg_base);
 }
 
 =head2 xdg_config_home
@@ -448,31 +416,28 @@ my $xdg_config_home = sub {
     }
     elsif ( $^O eq "MSWin32" )
     {
-        push( @dirs, File::Spec->catdir( $ENV{APPDATA}, @cfg_base ) );
+        @dirs = ( File::Spec->catdir( $ENV{APPDATA}, @cfg_base ) );
     }
     else
     {
-        push( @dirs, File::Spec->catdir( File::HomeDir->my_home(), ".config", @cfg_base ) )
-          if ($haveFileHomeDir);
+        $haveFileHomeDir and @dirs = ( File::Spec->catdir( File::HomeDir->my_home(), ".config", @cfg_base ) );
     }
 
-    return @dirs;
+    @dirs;
 };
 
 sub xdg_config_home
 {
     my @cfg_base = @_;
     1 < scalar(@cfg_base)
-      and croak "xdg_config_home(;\$), not xdg_config_home("
-      . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
-    return &{$xdg_config_home}(@cfg_base);
+      and croak "xdg_config_home(;\$), not xdg_config_home(" . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
+    $xdg_config_home->(@cfg_base);
 }
 
 my ( @extensible_bases, @pure_bases );
 push( @extensible_bases,
-      $system_cfg_dir, $desktop_cfg_dir, $local_cfg_dir,  $singleapp_cfg_dir,
-      $core_cfg_dir,   $site_cfg_dir,    $vendor_cfg_dir, $here_cfg_dir,
-      $user_cfg_dir,   $xdg_config_home );
+    $system_cfg_dir, $desktop_cfg_dir, $local_cfg_dir, $singleapp_cfg_dir, $core_cfg_dir,
+    $site_cfg_dir,   $vendor_cfg_dir,  $here_cfg_dir,  $user_cfg_dir,      $xdg_config_home );
 push( @pure_bases, 3 );
 
 =head2 config_dirs
@@ -489,8 +454,7 @@ sub config_dirs
 {
     my @cfg_base = @_;
     1 < scalar(@cfg_base)
-      and croak "config_dirs(;\$), not config_dirs("
-      . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
+      and croak "config_dirs(;\$), not config_dirs(" . join( ",", ("\$") x scalar(@cfg_base) ) . ")";
     my @dirs = ();
 
     my $pure_idx = 0;
@@ -498,12 +462,12 @@ sub config_dirs
     {
         my $pure;
         $pure_idx <= $#pure_bases and $idx == $pure_bases[$pure_idx] and $pure = ++$pure_idx;
-        push( @dirs, &{ $extensible_bases[$idx] }( ( $pure ? () : @cfg_base ) ) );
+        push( @dirs, $extensible_bases[$idx]->( ( $pure ? () : @cfg_base ) ) );
     }
 
     @dirs = grep { -d $_ && -r $_ } uniq(@dirs);
 
-    return @dirs;
+    @dirs;
 }
 
 =head2 _plug_dir_source
@@ -525,17 +489,16 @@ when C<$dir_src> is not a code ref.
 
 sub _plug_dir_source
 {
-    my ($dir_source, $pure) = @_;
+    my ( $dir_source, $pure ) = @_;
 
     $dir_source or return;
     "CODE" eq ref $dir_source or return;
 
-    push(@extensible_bases, $dir_source);
-    $pure and push(@pure_bases, $#extensible_bases);
-    return scalar @extensible_bases;
+    push( @extensible_bases, $dir_source );
+    $pure and push( @pure_bases, $#extensible_bases );
+    1;
 }
 
-
 =head1 AUTHOR
 
 Jens Rehsack, C<< <rehsack at cpan.org> >>
@@ -543,7 +506,7 @@ Jens Rehsack, C<< <rehsack at cpan.org> >>
 =head1 BUGS
 
 Please report any bugs or feature requests to
-C<bug-file-configdir at rt.cpan.org>, or through the web interface at
+C<bug-File-ConfigDir at rt.cpan.org>, or through the web interface at
 L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=File-ConfigDir>.
 I will be notified, and then you'll automatically be notified of progress
 on your bug as I make changes.
@@ -585,7 +548,7 @@ and remind about C</usr/local/etc>.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2010-2014 Jens Rehsack.
+Copyright 2010-2015 Jens Rehsack.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
